﻿/*
* Description: An Adobe Illustrator script that export each layer as a separate ai file. For Illustrator CS4 and newer.
* License: This work is licensed under a Creative Commons Attribution-NoDerivatives 4.0 International License. (https://creativecommons.org/licenses/by-nd/4.0/)

* Copyright (c) 2011, Michel Simons
* http://www.no-nonsens.nl
* version 1.12 CS3
* November 7th 2011
* August 1th 2017

Under the following terms:

Attribution:
You must give appropriate credit, provide a link to the license, and indicate if changes were made. You may do so in any reasonable manner, but not in any way that suggests the licensor endorses you or your use.

NoDerivatives:
If you remix, transform, or build upon the material, you may not distribute the modified material.

No additional restrictions:
You may not apply legal terms or technological measures that legally restrict others from doing anything the license permits.


Notices:
You do not have to comply with the license for elements of the material in the public domain or where your use is permitted by an applicable exception or limitation.
No warranties are given. The license may not give you all of the permissions necessary for your intended use. For example, other rights such as publicity, privacy, or moral rights may limit how you use the material.
*/

// ********************************************************************************************************
// **                                                                                                    **
// ** initialize variables and start dialog                                                              **
// **                                                                                                    **
// ********************************************************************************************************
#target illustrator

if (app.name != "Adobe Illustrator") { alert ('This script is created for Adobe Illustrator, it is not developed for or tested on ' + app.name + '. Results can be unexpected and the script most likely will fail on execution.' ); }

var ScriptVersion = "1.12 CS3";

if (app.documents.length < 1) 
   {
    alert('Please open a document before running this script.', 'Error running SaveDrawingToLayers.jsx');
   }
    else
{
var export_to_dialog_window; // make window global
var CurrentDoc = app.activeDocument;
//var CurrentDoc_artboard = CurrentDoc.artboards[0].artboardRect;	
var PrefixCounter = 0; // prefix for layer names start with 0
var ContinueNumbering = 1; // 1 = number 1-2-3 // 2 = use real layer numbers // 3 = do not use prefix. Value set by radiobutton

var msis_dialog_base_path = "";    // default output folder
var unlocked_layers_count = 0; // number of unlocked layers in layer palette for progress bar
var msis_message_1 = "\nThis script will export a single Illustrator document for every UNLOCKED layer in this document. "+
                     "Locked layers will NOT be included in the layer export process. " +
                     "Note that running this script will change layer visibility for your current document. You might want to save your document first."; 
var msis_message_2 = "SaveDrawingToLayersAI.jsx\n\n" +
                     "All copyright (c) 2011 No-Nonsens inc. reserved.\n" +
                     "written by Michel Simons / No-Nonsens inc.\n" +
					 "http://www.no-nonsens.nl\n\n" +
					 "released under Creative Commons Attribution-NoDerivatives 4.0 International License.";
					 
					 
					 
var OriginalPathName = app.activeDocument.path;
var OriginalFileName = app.activeDocument.name;		
		 
					 
if (CurrentDoc.path != "") 
{
  msis_dialog_base_path = app.activeDocument.path.fsName;
}


// count unlocked layers for progress bar
for(var i=0; i<CurrentDoc.layers.length; i++) 
 {
  if (CurrentDoc.layers[i].locked == false) { unlocked_layers_count++; }
 } 
  
// run dialog screen
 export_to_file_dialog();

}



// ********************************************************************************************************
// **                                                                                                    **
// ** copy each layer to new document and save new document as ai file                                   **
// **                                                                                                    **
// ********************************************************************************************************



function ProcessLayer(name, id, LayerCounter, ExportPath ) 
  {
  
    var myFilename = name;
	 
	 
   // remove all of the layers except the current one
    try
    {
	   for (var z = CurrentDoc.layers.length-1; z >= 0; z--) 	 
	    {
		 if ( z != id)
 		 {
		  if (CurrentDoc.layers[z].locked == true) {CurrentDoc.layers[z].locked = false;}
		  if (CurrentDoc.layers[z].visible == false) {CurrentDoc.layers[z].visible = true;}
 		  CurrentDoc.layers[z].remove();
		 }
		}		
	 }
   catch(err)
	{
     alert("An error occured Removing Layers .\n(" + err + ").\n\nFunction: ProcessLayers: Remove Layers.");
    }	
	 
	 
   try
    {
	if (ContinueNumbering != 3) // option is not 3 so a prefix needs to be added
	   {
	    // construct filename with layer number as prefix
        if ( ContinueNumbering == 1) { LayerCounter = id + 1; } // 2 so use layer id number instead of continous numbering
	    if ( LayerCounter < 10 )                      { myFilename = "00" + LayerCounter + " " + myFilename; }
	    if ( LayerCounter > 9 && LayerCounter < 100 ) { myFilename = "0" + LayerCounter + " " + myFilename; }
	    if ( LayerCounter > 99)                       { myFilename = LayerCounter + " " + myFilename; }
	   }
	
	// check layer name for characters not allowed in file/folder names and convert them to underscores
	myFilename = myFilename.replace(/\*/g, "_");      // convert *
    myFilename = myFilename.replace(/\[/g, "_");      // convert [
    myFilename = myFilename.replace(/\]/g, "_");      // convert ]
    myFilename = myFilename.replace(/\</g, "_");      // convert <
    myFilename = myFilename.replace(/\>/g, "_");      // convert >
    myFilename = myFilename.replace(/\=/g, "_");      // convert =
    myFilename = myFilename.replace(/\+/g, "_");      // convert +
    myFilename = myFilename.replace(/\'/g, "_");      // convert '
    myFilename = myFilename.replace(/\"/g, "_");      // convert "
    myFilename = myFilename.replace(/\\/g, "_");      // convert \ 
    myFilename = myFilename.replace(/\//g, "_");      // convert /
    myFilename = myFilename.replace(/\,/g, "_");      // convert ,
    myFilename = myFilename.replace(/\./g, "_");      // convert .
    myFilename = myFilename.replace(/\:/g, "_");      // convert :
    myFilename = myFilename.replace(/\;/g, "_");      // convert ;
    myFilename = myFilename.replace(/\?/g, "_");      // convert ?
    myFilename = myFilename.replace(/\|/g, "_");      // convert |	
    // myFilename = myFilename.replace(/ /g, "_")       // convert spaces
	
	
	
   // init Folder Path
   var myPath = new Folder ( ExportPath );	

   // init Folder Path
   var myFile = new File ( myPath + "/" + myFilename + ".ai" );

	}
   catch(err)
	{
     alert("An error occured Saving Document " + path +".\n(" + err + ").\n\nFunction: ProcessLayers: Creating file name.\nYou might need to check this document manually after the script has finished.");
    }		
	
	 

  try
    {
	
	
    export_to_dialog_window.ProgressLabel.text = 'Exporting layer ' + PrefixCounter + ' out of ' + unlocked_layers_count + ' unlocked layers.';
    export_to_dialog_window.ProgressProgressBar.value = (100 / unlocked_layers_count) * PrefixCounter;
    //export_to_dialog_window.update(); 	

    var saveOptions = new IllustratorSaveOptions();

    if ( export_to_dialog_window.myCompressedSettings.value == false )  { saveOptions.compressed = false; }          else { saveOptions.compressed = true; }
    if ( export_to_dialog_window.myEmbedLinkedSettings.value == false ) { saveOptions.embedLinkedFiles = false; }    else { saveOptions.embedLinkedFiles = true; }
    if ( export_to_dialog_window.myICCSettings.value == false )         { saveOptions.embedICCProfile = false; }     else { saveOptions.embedICCProfile = true; }
    if ( export_to_dialog_window.myPDFCompSettings.value == false )     { saveOptions.myPDFCompSettings = false; }   else { saveOptions.myPDFCompSettings = true; }

    // saveOptions.saveMultipleArtboards = true;
    // saveOptions.artboardRange = "1";  
  
   CurrentDoc.saveAs( myFile , saveOptions ); 

   //CurrentDoc.close( SaveOptions.DONOTSAVECHANGES );

	// wait for the new file to export and close before continue.
	// A callback function (if possible) will be better than a while loop for sure.
	//while (app.documents.length > 1) 
	// {
	//   continue;
	// } 

  }
   catch(err)
	{
     alert("An error occured Saving Document " + path +".\n(" + err + ").\n\nFunction: exportFileToAI.\nYou might need to check this document manually after the script has finished.");
    }	 
	
    // restore all removed layers by using application undo
  	app.undo(); 	 
	 
}
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************
// *******************************************************************************************************************************


// ********************************************************************************************************
// **                                                                                                    **
// ** building and showing main dialog                                                                   **
// **                                                                                                    **
// ********************************************************************************************************
function export_to_file_dialog() {
   
   // Export dialog
    export_to_dialog_window = new Window('dialog', 'Export each layer to a new Illustrator document  -   v' + ScriptVersion); 

    // PANEL with usage instructions
    export_to_dialog_window.MessagePanel = export_to_dialog_window.add('panel', undefined, 'Export Layers as AI Documents'); 
    // GROUP
    var Message_Group = export_to_dialog_window.MessagePanel.add('group', undefined, '')
    Message_Group.orientation = 'column';
    Message_Group.alignment = [ScriptUI.Alignment.LEFT, ScriptUI.Alignment.TOP]
	// Labels
    var Message_Group_label_1 = Message_Group.add('statictext', undefined,  msis_message_1, {multiline:true}); 
    Message_Group_label_1.size = [500,80];

	



    // PANEL with Prefix radio buttons
	export_to_dialog_window.PrefixPanel =  export_to_dialog_window.add ('panel', undefined, 'File Name Prefix Settings:');
    // GROUP
	var Radio_Group = export_to_dialog_window.PrefixPanel.add('group', undefined, '')
	Radio_Group.alignChildren = "left";
	Radio_Group.size = [500,85];
	Radio_Group.orientation = 'column';
	// Radio Boxes
	var RadioBox1 = Radio_Group.add ("radiobutton", undefined, "Use layer number as file name prefix (also count locked layers).");
	var RadioBox2 = Radio_Group.add ("radiobutton", undefined, "Use layer number as file name prefix (do not count locked layers).");
	var RadioBox3 = Radio_Group.add ("radiobutton", undefined, "Do not use a file name prefix.");
	RadioBox1.value = true;

    // PANEL with Export Options
	export_to_dialog_window.OptionsPanel =  export_to_dialog_window.add ('panel', undefined, 'AI Export Options:');
	// GROUP
	Options_Group2 = export_to_dialog_window.OptionsPanel.add('group', undefined, '')
	Options_Group2.alignChildren = "left";
	Options_Group2.size = [500,25];
	Options_Group2.orientation = 'row';
	
	export_to_dialog_window.myCompressedSettings = Options_Group2.add ("checkbox", undefined, "Compressed");
    export_to_dialog_window.myCompressedSettings.value = true;	
	
	export_to_dialog_window.myEmbedLinkedSettings = Options_Group2.add ("checkbox", undefined, "Embed Linked Files");
    export_to_dialog_window.myEmbedLinkedSettings.value = true;	

	export_to_dialog_window.myPDFCompSettings = Options_Group2.add ("checkbox", undefined, "PDF Compatible");
    export_to_dialog_window.myPDFCompSettings.value = true;
	
	export_to_dialog_window.myICCSettings = Options_Group2.add ("checkbox", undefined, "Include ICC profile");
    export_to_dialog_window.myICCSettings.value = true; 
	
    // PANEL with Export Location
    export_to_dialog_window.ExportPanel = export_to_dialog_window.add('panel', undefined, 'File Location Settings:'); 
    // GROUP
    var Export_Group = export_to_dialog_window.ExportPanel.add( 'group', undefined, '') 
    Export_Group.orientation = 'row';
    Export_Group.alignment = [ScriptUI.Alignment.LEFT, ScriptUI.Alignment.TOP];
	Export_Group.size = [500,30];
	// Label
	var ExportLabel = Export_Group.add('statictext', undefined, 'Output folder:'); 
    ExportLabel.size = [ 100,20 ];
	// Edit
    var ExportEdit = Export_Group.add('edittext', undefined, msis_dialog_base_path); 
    ExportEdit.size = [ 280,20 ];
    // Choose Button
    var ExportChooseButton = Export_Group.add('button', undefined, 'Choose ...' );
    ExportChooseButton.onClick = function() { msis_dialog_base_path = Folder.selectDialog().fsName; ExportEdit.text = msis_dialog_base_path; }
	
	
    // PANEL with Progressbar
    export_to_dialog_window.ProgressPanel = export_to_dialog_window.add('panel', undefined, 'Status:'); 	
    // GROUP
    var Progress_Group = export_to_dialog_window.ProgressPanel.add('group', undefined, '')
    Progress_Group.orientation = 'column';
    Progress_Group.alignment = [ScriptUI.Alignment.LEFT, ScriptUI.Alignment.TOP]
	Progress_Group.size = [500,5];		
    // progressbar
	export_to_dialog_window.ProgressProgressBar = export_to_dialog_window.ProgressPanel.add( 'progressbar', undefined, 0, 100 );
    export_to_dialog_window.ProgressProgressBar.size = [480,10];
	// label
	export_to_dialog_window.ProgressLabel = export_to_dialog_window.ProgressPanel.add('statictext', undefined, 'This document contains ' + unlocked_layers_count + ' unlocked layer(s) to export.' ); 
	export_to_dialog_window.ProgressLabel.size = [ 480,20 ];



    // Buttons don't have a PANEL
    // GROUP
    Button_Group = export_to_dialog_window.add('group', undefined, ''); 
    Button_Group.orientation = 'row'

    Button_Group.cancelBtn = Button_Group.add('button', undefined, 'Cancel', {name:'cancel'});
    Button_Group.cancelBtn.onClick = function() { msis_file_dlg.close() };


    Button_Group.okBtn = Button_Group.add('button', undefined, 'Export', {name:'ok'});
    Button_Group.okBtn.onClick = function() 
	             { msis_dialog_base_path = ExportEdit.text; 
				   Button_Group.okBtn.enabled = false; 
				   Button_Group.cancelBtn.enabled = false;
	               Button_Group.aboutBtn.enabled = false;				   
				   ExportEdit.enabled = false;
				   ExportChooseButton.enabled = false;
				   RadioBox1.enabled = false;
				   RadioBox2.enabled = false;				   
				   RadioBox3.enabled = false;
				   
				   export_to_dialog_window.myCompressedSettings.enabled = false; 				   
				   export_to_dialog_window.myEmbedLinkedSettings.enabled = false; 
				   export_to_dialog_window.myPDFCompSettings.enabled = false;
				   export_to_dialog_window.myICCSettings.enabled = false;
				   
				   if (RadioBox1.value == true)  {ContinueNumbering = 1;}
				   if (RadioBox2.value == true)  {ContinueNumbering = 2;}	
				   if (RadioBox3.value == true)  {ContinueNumbering = 3;}				   
				   msis_run_export();   
				   
				   Button_Group.okBtn.enabled = true; 
	               Button_Group.cancelBtn.enabled = true;
	               Button_Group.aboutBtn.enabled = true;
				   ExportEdit.enabled = true;
	               ExportChooseButton.enabled = true;
                   RadioBox1.enabled = true;
	               RadioBox2.enabled = true;				   
	               RadioBox3.enabled = true;
				   export_to_dialog_window.myCompressedSettings.enabled = true; 				   
				   export_to_dialog_window.myEmbedLinkedSettings.enabled = true; 
				   export_to_dialog_window.myPDFCompSettings.enabled = true;
				   export_to_dialog_window.myICCSettings.enabled = true;
				   
				 };
				 
				 
			 
				 
    Button_Group.aboutBtn = Button_Group.add('button', undefined, 'About', {name:'about'});
    Button_Group.aboutBtn.onClick = function() { alert(msis_message_2); };				 
				 

    export_to_dialog_window.show();
}



// ********************************************************************************************************
// **                                                                                                    **
// ** execute main routine when clicking at the Export button                                            **
// **                                                                                                    **
// ********************************************************************************************************
function msis_run_export( )
 {
  // path specified
  if ( msis_dialog_base_path == '' )
     {
      alert ('Invalid output folder.\n\nPlease specify an export location (output folder) for new documents.');
	  return;
	 }

  // check if path exists
  var CheckPath = Folder(msis_dialog_base_path);
  if ( CheckPath.exists == 0)
	 {
      alert ('Invalid output folder.\n\nPlease use the Choose Button to select your output folder.');
	  return;
	 }  

	 
	 
	// go through each layer of the current document 
    for (var j=0; j < CurrentDoc.layers.length; j++) 
      {
	   app.activeDocument = CurrentDoc;
       					   			
	   if ( j>0 ) { CurrentDoc.layers[j-1].visible = true; }
	       CurrentDoc.layers[j].visible = true;
	       CurrentDoc.activeLayer = CurrentDoc.layers[j];

    	   // skip locked layers
	       if (CurrentDoc.layers[j].locked == false)
   	          {
			   PrefixCounter++;	  
			   export_to_dialog_window.ProgressLabel.text = 'Exporting layer ' + PrefixCounter + ' out of ' + unlocked_layers_count + ' layers.)';
               export_to_dialog_window.ProgressProgressBar.value = (100 / unlocked_layers_count) * PrefixCounter;
			   
   //            export_to_dialog_window.update(); 
			   
     		   ProcessLayer(CurrentDoc.activeLayer.name, j, PrefixCounter, Folder(msis_dialog_base_path) );
			  } // end locked layers
	 
	  }

 
	  
	  
    // create backup filename and save drawing as backup
	var TheFileName = OriginalFileName.substr(0, OriginalFileName.lastIndexOf('.'));
	var ExtensionArray = OriginalFileName.split('.');
	TheFileExtension = ExtensionArray[ExtensionArray.length-1];
	var NewFileName = ( OriginalPathName + '/' + TheFileName + ' [BACKUP]' + '.' + TheFileExtension);

	export_to_dialog_window.ProgressLabel.text = 'Saving copy of drawing (backup): ' + TheFileName + ' [BACKUP].' + TheFileExtension;
    //export_to_dialog_window.update(); 	
	
	var saveOptions = new IllustratorSaveOptions();

    if ( export_to_dialog_window.myCompressedSettings.value == false )  { saveOptions.compressed = false; }          else { saveOptions.compressed = true; }
    if ( export_to_dialog_window.myEmbedLinkedSettings.value == false ) { saveOptions.embedLinkedFiles = false; }    else { saveOptions.embedLinkedFiles = true; }
    if ( export_to_dialog_window.myICCSettings.value == false )         { saveOptions.embedICCProfile = false; }     else { saveOptions.embedICCProfile = true; }
    if ( export_to_dialog_window.myPDFCompSettings.value == false )     { saveOptions.myPDFCompSettings = false; }   else { saveOptions.myPDFCompSettings = true; }

	// saveOptions.saveMultipleArtboards = true;
    // saveOptions.artboardRange = "1";  
    CurrentDoc.saveAs( File (NewFileName) , saveOptions );

	 
    // show message that export is finished
    alert ('Exported : ' + unlocked_layers_count + ' layers to: ' + msis_dialog_base_path +"\n\nNote: your current file has been (re)saved as: \n" + TheFileName + ' [BACKUP]' + '.' + TheFileExtension	); 	  
    msis_file_dlg.close();
}	  